package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.params.GameAccountUpdateParam;
import com.zanxiang.manage.domain.vo.GameAccountVO;
import com.zanxiang.manage.service.GameAppletService;
import com.zanxiang.mybatis.entity.GameApplet;
import com.zanxiang.mybatis.mapper.GameAppletMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * @author : lingfeng
 * @time : 2022-07-08
 * @description : 微信小游戏(小程序)
 */
@Slf4j
@Service
public class GameAppletServiceImpl extends ServiceImpl<GameAppletMapper, GameApplet> implements GameAppletService {

    /**
     * 获取账号信息配置
     *
     * @param id : 游戏id
     * @return : 账号配置信息
     */
    @Override
    public GameAccountVO getGameAccount(Long id) {
        GameApplet gameApplet = super.getOne(new LambdaQueryWrapper<GameApplet>()
                .eq(GameApplet::getGameId, id));
        if (gameApplet == null) {
            GameAccountVO vo = new GameAccountVO();
            vo.setGameId(id);
            return vo;
        }
        return BeanUtils.copy(gameApplet, GameAccountVO.class);
    }

    /**
     * 账号配置信息更新或者添加
     *
     * @param param : 更新参数
     * @return : 账号配置信息
     */
    @Override
    public Boolean gameAccountAddOrUpdate(GameAccountUpdateParam param) {
        GameApplet gameApplet;
        if (param.getId() == null) {
            gameApplet = new GameApplet();
            gameApplet.setCreateTime(LocalDateTime.now());
            gameApplet.setGameId(param.getGameId());
        } else {
            gameApplet = super.getById(param.getId());
        }
        gameApplet.setAppId(param.getAppId());
        gameApplet.setGhId(param.getGhId());
        gameApplet.setAccount(param.getAccount());
        gameApplet.setPassword(param.getPassword());
        gameApplet.setAdminVx(param.getAdminVx());
        gameApplet.setCompany(param.getCompany());
        gameApplet.setAppSecret(param.getAppSecret());
        gameApplet.setUpdateTime(LocalDateTime.now());
        return super.saveOrUpdate(gameApplet);
    }
}
