package com.zanxiang.sdk.service.Impl;

import com.alipay.easysdk.kernel.util.Signer;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zanxiang.common.enums.CpStatusEnum;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.enums.ResEnum;
import com.zanxiang.common.utils.MD5Util;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.mybatis.entity.*;
import com.zanxiang.sdk.common.util.HttpUtil;
import com.zanxiang.sdk.domain.dto.PlatformOrderDTO;
import com.zanxiang.sdk.domain.mapper.OrderCompleteCreateMapper;
import com.zanxiang.sdk.service.*;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Locale;
import java.util.Objects;
import java.util.TreeMap;

/**
 * 订单履约逻辑
 *
 * @author xufeng
 * @date 2022/6/27 11:17
 */
@Component
@Slf4j
@Service
@Transactional(rollbackFor = {RuntimeException.class, Exception.class})
public class PerformOrderServiceImpl implements PerformOrderService {
    protected final Logger logger = LoggerFactory.getLogger(PlatformOrderServiceImpl.class);

    @Autowired
    private PlatformOrderService platformOrderService;

    @Autowired
    private LogPayCpService logPayCpService;

    @Autowired
    private OrderCompleteCreateMapper orderCompleteMapper;

    @Autowired
    private GameUserRoleService gameUserRoleService;

    @Autowired
    private GameUserService gameUserService;

    @Autowired
    private UserService userService;

    @Autowired
    private OrderCompleteService orderCompleteService;

    public Boolean pushCp(PlatformOrderDTO orderInfo) {
        if (orderInfo.getCpStatus() == CpStatusEnum.SUCCESS.getCode()) {
            return true;
        }
        String orderId = orderInfo.getId();
        //todo 待完善
        String app_key = "111";
        String cpPaybackUrl = "https://baidu.com";

        HashMap<String, String> map = new HashMap<>();
        map.put("app_id", String.valueOf(orderInfo.getGameId()));
        map.put("cp_order_id", orderInfo.getCpOrderId());
        map.put("mem_id", orderInfo.getMgUserId());
        map.put("order_status", String.valueOf(orderInfo.getStatus()));
        map.put("pay_time", String.valueOf(orderInfo.getPayTime()));
        map.put("product_id", orderInfo.getProductId());
        map.put("product_price", String.valueOf(orderInfo.getAmount()));
        map.put("ext", orderInfo.getExt());
        map.put("app_key", app_key);
        TreeMap<String, String> keySort = new TreeMap<>(map);
        //拼接字符串
        String param = Signer.getSignCheckContent(keySort);
        //拼接app_key
        param += "&app_key=" + app_key;
        //生成sign
        String sign = MD5Util.MD5Encode(param, "UTF-8");
        map.remove("app_key");
        map.put("sign", sign);
        //拼接sign请求
        param += "&sign=" + sign;
        try {
            Integer i = orderInfo.getCpNotifyCnt() + 1;
            String s = HttpUtil.postData(cpPaybackUrl, param);
            Boolean pushSuccess = false;
            if (s.trim().toLowerCase(Locale.ROOT).equals(ResEnum.SUCCESS.getMsg())) {
                pushSuccess = true;
                //更新订单状态
                Order order = new Order();
                order.setId(orderId);
                order.setCpNotifyCnt(i);
                order.setLastCpNotifyTime(LocalDateTime.now());
                order.setCpStatus(pushSuccess ? CpStatusEnum.SUCCESS.getCode() : CpStatusEnum.FAILLE.getCode());
                platformOrderService.updateById(order);
            }
            LogPayCp logPayCp = new LogPayCp();
            logPayCp.setOrderId(orderId);
            logPayCp.setCpOrderId(orderInfo.getCpOrderId());
            logPayCp.setStatus(orderInfo.getStatus());
            logPayCp.setCpStatus(pushSuccess ? CpStatusEnum.SUCCESS.getCode() : CpStatusEnum.FAILLE.getCode());
            logPayCp.setCpPaybackUrl(cpPaybackUrl);
            logPayCp.setParams(param);
            logPayCp.setExt(orderInfo.getExt());
            logPayCp.setNotifyCnt(i);
            logPayCpService.save(logPayCp);
        } catch (Exception e) {
            logger.error("订单:{} 通知cp方失败 e:{}", orderId, e);
        }
        return false;
    }

    @Override
    public Boolean createOrderComplete(PlatformOrderDTO orderInfo) {
        try {
            OrderComplete orderComplete = orderCompleteMapper.info(orderInfo.getId());
            orderComplete.setOrderDate(orderInfo.getCreateTime());
            //todo 推广相关信息
            orderComplete.setAgentId("");
            orderComplete.setAgentAccount("");
            orderComplete.setAgentMedia("");
            orderComplete.setAgentAdvert("");

            //todo 广告通知状态
            orderComplete.setAdvertNotifyStatus(0);
            orderComplete.setAdvertNotifyType(0);

            OrderComplete complete = orderCompleteService.getOne(new LambdaQueryWrapper<OrderComplete>().eq(OrderComplete::getOrderId, orderInfo.getId()));
            if (Objects.isNull(complete)) {
                orderCompleteMapper.insert(orderComplete);
                logger.info("订单:{} 生成宽表数据成功", orderInfo.getId());
            } else {
                OrderComplete pa = new OrderComplete();
                pa.setOrderId(complete.getOrderId());
                orderCompleteService.update(pa, new UpdateWrapper<OrderComplete>().setEntity(orderComplete));
                logger.info("订单:{} 宽表数据更新成功", orderInfo.getId());
            }
            return true;
        } catch (Exception e) {
            logger.error("订单:{} 生成宽表数据失败 e:{}", orderInfo.getId(), e);
        }
        return false;
    }

    @Override
    public Boolean checkIsFirstRecharge(PlatformOrderDTO orderInfo) {
        try {
            Order one = platformOrderService.getOne(new QueryWrapper<Order>().lambda().eq(Order::getUserId, orderInfo.getUserId()).ne(Order::getId, orderInfo.getId()).last("limit 1"));
            if (Objects.isNull(one)) {
                Order order = new Order();
                order.setId(order.getId());
                order.setIsFirstRecharge(1);
                platformOrderService.updateById(order);
                logger.info("订单:{} 验证订单是否为首充成功", orderInfo.getId());
                return true;
            }
        } catch (Exception e) {
            logger.error("订单:{} 验证订单是否为首充失败 e:{}", orderInfo.getId(), e);
        }
        return true;
    }

    @Override
    public Boolean userRechargeTotal(PlatformOrderDTO orderInfo) {
        try {
            //订单取消情况 做减法
            float amount = orderInfo.getAmount().floatValue();
            int num = 1;
            if (orderInfo.getStatus().equals(OrderStateEnum.CANCEL.getCode())) {
                amount = -amount;
                num = -1;
            }
            //游戏用户角色统计更新
            if (StringUtils.isNotEmpty(orderInfo.getRoleId())) {
                GameUserRole gameUserRole = new GameUserRole();
                gameUserRole.setRoleId(orderInfo.getRoleId());
                gameUserRoleService.update(gameUserRole, new UpdateWrapper<GameUserRole>().lambda()
                        .set(GameUserRole::getLastRechargeTime, orderInfo.getPayTime())
                        .setSql("recharge_count=recharge_count+" + num)
                        .setSql("recharge_money=recharge_money+" + amount)
                        .setEntity(gameUserRole)
                );
            } else {
                logger.info("订单:{} 用户玩家角色RoleId为空，不进行gameUserRole充值统计汇总", orderInfo.getId());
            }
            //游戏用户统计更新
            if (StringUtils.isNotEmpty(orderInfo.getMgUserId())) {
                GameUser gameUser = new GameUser();
                gameUser.setId(Long.valueOf(orderInfo.getMgUserId()));
                gameUserService.update(gameUser, new UpdateWrapper<GameUser>().lambda()
                        .set(GameUser::getLastRechargeTime, orderInfo.getPayTime())
                        .setSql("recharge_count=recharge_count+" + num)
                        .setSql("recharge_money=recharge_money+" + amount)
                        .setEntity(gameUser)
                );
            } else {
                logger.info("订单:{} 用户玩家MgUserId为空，不进行gameUser充值统计汇总", orderInfo.getId());
            }
            //用户统计更新
            if (StringUtils.isNotEmpty(orderInfo.getUserId())) {
                User user = new User();
                user.setId(Long.valueOf(orderInfo.getUserId()));
                userService.update(user, new UpdateWrapper<User>().lambda()
                        .set(User::getLastRechargeTime, orderInfo.getPayTime())
                        .setSql("recharge_count=recharge_count+" + num)
                        .setSql("recharge_money=recharge_money+" + amount)
                        .setEntity(user)
                );
            } else {
                logger.info("订单:{} 用户UserId为空，不进行User充值统计汇总", orderInfo.getId());
            }
            return true;
        } catch (Exception e) {
            logger.error("订单:{} 用户充值统计逻辑操作失败 e:{}", orderInfo.getId(), e);
        }
        return false;
    }
}
